@@ -27,23 +27,28 @@
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef _MSC_VER
+#ifdef WIN32
+# include <winsock2.h>
 # include <windows.h>
 # define inline __inline
-#else /* !_MSC_VER */
+#else /* !WIN32 */
 #include <unistd.h>
 #endif
 
 #include <mysql.h>
 
-#include <wzd_backend.h>
-#include <libwzd-auth/wzd_md5.h>
-#include <libwzd-auth/wzd_md5crypt.h>
-#include <wzd_debug.h>
+#include <libwzd-auth/wzd_auth.h>
+
+#include <libwzd-core/wzd_backend.h>
+#include <libwzd-core/wzd_log.h>
+
+#include <libwzd-core/wzd_debug.h>
 
 #include "libmysql.h"
 
-#define MYSQL_BACKEND_VERSION   121
+#define MYSQL_BACKEND_VERSION   122
+
+#define MYSQL_LOG_CHANNEL       (RESERVED_LOG_CHANNELS+16)
 
 /* IMPORTANT needed to check version */
 BACKEND_NAME(mysql);
@@ -74,7 +79,7 @@ static gid_t * wzd_mysql_get_group_list(void);
 
 void _wzd_mysql_error(const char *filename, const char  *func_name, int line)/*, const char *error)*/
 {
-  fprintf(stderr, "%s(%s):%d %s\n", filename, func_name, line, mysql_error(&mysql));
+  out_log(MYSQL_LOG_CHANNEL, "%s(%s):%d %s\n", filename, func_name, line, mysql_error(&mysql));
 }
 
 static int wzd_parse_arg(const char *arg)
@@ -178,9 +183,7 @@ uid_t FCN_VALIDATE_LOGIN(const char *login, wzd_user_t * user)
 uid_t FCN_VALIDATE_PASS(const char *login, const char *pass, wzd_user_t * user)
 {
   char *query;
-  char * cipher;
   uid_t uid;
-  char buffer[128];
 
   if (!wzd_mysql_check_name(login)) return (uid_t)-1;
 
@@ -236,19 +239,17 @@ uid_t FCN_VALIDATE_PASS(const char *login, const char *pass, wzd_user_t * user)
 
     if (strlen(stored_pass) == 0)
     {
-      fprintf(stderr,"WARNING: empty password field whould not be allowed !\n");
-      fprintf(stderr,"WARNING: you should run: UPDATE users SET userpass='%%' WHERE userpass is NULL\n");
+      out_log(MYSQL_LOG_CHANNEL,"WARNING: empty password field whould not be allowed !\n");
+      out_log(MYSQL_LOG_CHANNEL,"WARNING: you should run: UPDATE users SET userpass='%%' WHERE userpass is NULL\n");
       return uid; /* passworldless login */
     }
 
     if (strcmp(stored_pass,"%")==0)
       return uid; /* passworldless login */
 
-    cipher = (char*)md5_hash_r(pass, buffer, sizeof(buffer));
-    if (!cipher) return (uid_t)-1;
-
-    if (strncasecmp(cipher,stored_pass,32))
-      return (uid_t)-1;
+    if (check_auth(login, pass, stored_pass)==1)
+      return uid;
+    return (uid_t)-1;
 
   } /* else // user does not exist in table
     return -1;*/
@@ -373,6 +374,7 @@ wzd_user_t * FCN_GET_USER(uid_t uid)
   wzd_row_get_string(user->username, HARD_USERNAME_LENGTH, row, UCOL_USERNAME);
   wzd_row_get_string(user->userpass, MAX_PASS_LENGTH, row, UCOL_USERPASS);
   wzd_row_get_string(user->rootpath, WZD_MAX_PATH, row, UCOL_ROOTPATH);
+  wzd_row_get_string(user->tagline, MAX_TAGLINE_LENGTH, row, UCOL_TAGLINE);
   wzd_row_get_string(user->flags, MAX_FLAGS_NUM, row, UCOL_FLAGS);
   wzd_row_get_uint((unsigned int*)&user->max_idle_time, row, UCOL_MAX_IDLE_TIME);
   wzd_row_get_uint(&user->max_ul_speed, row, UCOL_MAX_UL_SPEED);
@@ -390,7 +392,7 @@ wzd_user_t * FCN_GET_USER(uid_t uid)
   /* Now get IP */
   user->ip_allowed[0][0] = '\0';
 
-  snprintf(query, 512, "SELECT UserIP.ip FROM UserIP,users WHERE users.uid='%d' AND users.ref=UserIP.ref", uid);
+  snprintf(query, 512, "SELECT userip.ip FROM userip,users WHERE users.uid='%d' AND users.ref=userip.ref", uid);
 
   if (mysql_query(&mysql, query) != 0) {
     free(query);
@@ -406,7 +408,7 @@ wzd_user_t * FCN_GET_USER(uid_t uid)
   i =0;
   while ( (row = mysql_fetch_row(res)) ) {
     if (i >= HARD_IP_PER_USER) {
-      fprintf(stderr,"Mysql: too many IP for user %s, dropping others\n",user->username);
+      out_log(MYSQL_LOG_CHANNEL,"Mysql: too many IP for user %s, dropping others\n",user->username);
       break;
     }
     wzd_row_get_string(user->ip_allowed[i], MAX_IP_LENGTH, row, 0 /* query asks only one column */);
@@ -418,7 +420,7 @@ wzd_user_t * FCN_GET_USER(uid_t uid)
 
   /* Now get Groups */
 
-  snprintf(query, 512, "SELECT groups.gid FROM groups,users,UGR WHERE users.uid='%d' AND users.ref=UGR.uref AND groups.ref=UGR.gref", uid);
+  snprintf(query, 512, "SELECT groups.gid FROM groups,users,ugr WHERE users.uid='%d' AND users.ref=ugr.uref AND groups.ref=ugr.gref", uid);
 
   if (mysql_query(&mysql, query) != 0) {
     free(query);
@@ -434,7 +436,7 @@ wzd_user_t * FCN_GET_USER(uid_t uid)
   i =0;
   while ( (row = mysql_fetch_row(res)) ) {
     if (i >= HARD_IP_PER_USER) {
-      fprintf(stderr,"Mysql: too many groups for user %s, dropping others\n",user->username);
+      out_log(MYSQL_LOG_CHANNEL,"Mysql: too many groups for user %s, dropping others\n",user->username);
       break;
     }
     if (wzd_row_get_uint(&j, row, 0 /* query asks only one column */)==0)
@@ -444,8 +446,8 @@ wzd_user_t * FCN_GET_USER(uid_t uid)
 
   mysql_free_result(res);
 
-  /* Now get Stats */
-  snprintf(query, 512, "SELECT bytes_ul_total,bytes_dl_total,files_ul_total,files_dl_total FROM Stats,users WHERE users.uid=%d AND users.ref=Stats.ref", uid);
+  /* Now get stats */
+  snprintf(query, 512, "SELECT bytes_ul_total,bytes_dl_total,files_ul_total,files_dl_total FROM stats,users WHERE users.uid=%d AND users.ref=stats.ref", uid);
 
   if (mysql_query(&mysql, query) != 0) {
     free(query);
@@ -534,7 +536,7 @@ wzd_group_t * FCN_GET_GROUP(gid_t gid)
   group->ip_allowed[0][0] = '\0';
 
   query = malloc(512);
-  snprintf(query, 512, "SELECT GroupIP.ip FROM GroupIP,groups WHERE groups.gid='%d' AND groups.ref=GroupIP.ref", gid);
+  snprintf(query, 512, "SELECT groupip.ip FROM groupip,groups WHERE groups.gid='%d' AND groups.ref=groupip.ref", gid);
 
   if (mysql_query(&mysql, query) != 0) {
     free(query);
@@ -550,9 +552,7 @@ wzd_group_t * FCN_GET_GROUP(gid_t gid)
   i =0;
   while ( (row = mysql_fetch_row(res)) ) {
     if (i >= HARD_IP_PER_GROUP) {
-#ifdef DEBUG
-      fprintf(stderr,"Mysql: too many IP for group %s, dropping others\n",group->groupname);
-#endif
+      out_log(MYSQL_LOG_CHANNEL,"Mysql: too many IP for group %s, dropping others\n",group->groupname);
       break;
     }
     wzd_row_get_string(group->ip_allowed[i], MAX_IP_LENGTH, row, 0 /* query asks only one column */);
@@ -664,7 +664,7 @@ static uid_t * wzd_mysql_get_user_list(void)
   MYSQL_RES   *res;
   MYSQL_ROW    row;
   uid_t * uid_list;
-  unsigned int index, i;
+  unsigned int index, i=0;
   my_ulonglong num_rows;
 
   query = malloc(512);
@@ -707,7 +707,7 @@ static gid_t * wzd_mysql_get_group_list(void)
   MYSQL_RES   *res;
   MYSQL_ROW    row;
   gid_t * gid_list;
-  unsigned int index, i;
+  unsigned int index, i=0;
   my_ulonglong num_rows;
 
   query = malloc(512);
