@@ -28,6 +28,7 @@
 #include <string.h>
 
 #ifdef _MSC_VER
+# include <winsock2.h>
 # include <windows.h>
 # define inline __inline
 #else /* !_MSC_VER */
@@ -36,14 +37,20 @@
 
 #include <libpq-fe.h>
 
-#include <wzd_backend.h>
-#include <libwzd-auth/wzd_md5.h>
-#include <libwzd-auth/wzd_md5crypt.h>
-#include <wzd_debug.h>
+#include <libwzd-auth/wzd_auth.h>
+
+#include <libwzd-core/wzd_backend.h>
+#include <libwzd-core/wzd_log.h>
+
+#include <libwzd-core/wzd_debug.h>
 
 #include "libpgsql.h"
 
-#define PGSQL_BACKEND_VERSION   101
+#define PGSQL_BACKEND_VERSION   103
+
+#define PGSQL_LOG_CHANNEL       (RESERVED_LOG_CHANNELS+17)
+
+#define PGSQL_DEFAULT_PORT      5432
 
 /* IMPORTANT needed to check version */
 BACKEND_NAME(pgsql);
@@ -52,6 +59,7 @@ BACKEND_VERSION(PGSQL_BACKEND_VERSION);
 
 PGconn * pgconn = NULL;
 static char *db_user, *db_passwd, *db_hostname, *db;
+static unsigned int db_port;
 
 /*static int wzd_parse_arg(const char *arg);*/ /* parse arg (login:password@hostname:table) */
 static int wzd_parse_arg(const char *arg);
@@ -74,14 +82,15 @@ static gid_t * wzd_pgsql_get_group_list(void);
 
 
 
-void _wzd_pgsql_error(const char *filename, const char  *func_name, int line)/*, const char *error)*/
+void _wzd_pgsql_error(const char *filename, const char  *func_name, int line)
 {
-  fprintf(stderr, "%s(%s):%d %s\n", filename, func_name, line, PQerrorMessage(pgconn));
+  out_log(PGSQL_LOG_CHANNEL, "%s(%s):%d %s\n", filename, func_name, line, PQerrorMessage(pgconn));
+
 }
 
 static int wzd_parse_arg(const char *arg)
 {
-  char *ptr;
+  char *ptr, *str_port;
   char * buffer;
 
   if (!arg) return -1;
@@ -97,9 +106,15 @@ static int wzd_parse_arg(const char *arg)
   db_hostname = strtok_r(NULL, ":\n", &ptr);
   if (!db_hostname) { free(buffer); return -1; }
 
-  db = strtok_r(NULL, "\n", &ptr);
+  db = strtok_r(NULL, ":\n", &ptr);
   if (!db) { free(buffer); return -1; }
 
+  str_port = strtok_r(NULL, "\n", &ptr);
+  if (str_port) {
+    db_port = strtoul(str_port,NULL,0);
+  } else
+    db_port = PGSQL_DEFAULT_PORT;
+
   return 0;
 }
 
@@ -109,19 +124,26 @@ int FCN_INIT(const char *arg)
   PGresult   *res;
 
   if ((wzd_parse_arg(arg)) != 0) {
+    out_log(PGSQL_LOG_CHANNEL, "%s(%s):%d could not parse arguments\n", __FILE__, __FUNCTION__, __LINE__);
     return -1;
   }
 
-  pgconn = PQsetdbLogin(db_hostname, /* db_port */ NULL, /* pgoptions */ NULL,
-      /* pgtty */ NULL, db, db_user, db_passwd);
+  {
+    wzd_string_t * str = str_allocate();
 
-  /** \todo XXX FIXME try using CLIENT_SSL for the last arg */
-/*  if (!mysql_real_connect(&mysql, db_hostname, db_user, db_passwd, db, 0, NULL, 0)) {*/
-  if (!pgconn) {
-#if 0
-    _wzd_mysql_error(__FILE__, __FUNCTION__, __LINE__);
-    mysql_close(&mysql);
-#endif
+    str_sprintf(str,"host=%s port=%u dbname=%s user=%s password=%s",
+        db_hostname, db_port, db, db_user, db_passwd);
+
+    pgconn = PQconnectdb(str_tochar(str));
+
+    str_deallocate(str);
+  }
+
+  if (!pgconn || PQstatus(pgconn)!=CONNECTION_OK) {
+    out_log(PGSQL_LOG_CHANNEL,"PG: could not connect to database %s on %s\n",db,db_hostname);
+    out_log(PGSQL_LOG_CHANNEL,"PG: please check connections and tables status\n");
+    _wzd_pgsql_error(__FILE__, __FUNCTION__, __LINE__);
+    if (pgconn) PQfinish(pgconn);
     return -1;
   }
 
@@ -129,12 +151,15 @@ int FCN_INIT(const char *arg)
   PQclear(res);
 
   if (!res) {
-    fprintf(stderr,"PG: could not connect to database %s on %s\n",db,db_hostname);
-    fprintf(stderr,"PG: please check connections and tables status\n");
+    out_log(PGSQL_LOG_CHANNEL,"PG: could not find expected data in database %s on %s\n",db,db_hostname);
+    out_log(PGSQL_LOG_CHANNEL,"PG: please check connections and tables status\n");
+    _wzd_pgsql_error(__FILE__, __FUNCTION__, __LINE__);
     PQfinish(pgconn);
     return -1;
   }
 
+  out_log(PGSQL_LOG_CHANNEL,"PG: backend version %d loaded\n",PGSQL_BACKEND_VERSION);
+
   return 0;
 }
 
@@ -191,9 +216,7 @@ uid_t FCN_VALIDATE_LOGIN(const char *login, wzd_user_t * user)
 uid_t FCN_VALIDATE_PASS(const char *login, const char *pass, wzd_user_t * user)
 {
   char *query;
-  char * cipher;
   uid_t uid;
-  char buffer[128];
   PGresult * res;
 
   if (!wzd_pgsql_check_name(login)) return (uid_t)-1;
@@ -244,19 +267,17 @@ uid_t FCN_VALIDATE_PASS(const char *login, const char *pass, wzd_user_t * user)
 
     if (strlen(stored_pass) == 0)
     {
-      fprintf(stderr,"WARNING: empty password field whould not be allowed !\n");
-      fprintf(stderr,"WARNING: you should run: UPDATE users SET userpass='%%' WHERE userpass is NULL\n");
+      out_log(PGSQL_LOG_CHANNEL,"WARNING: empty password field whould not be allowed !\n");
+      out_log(PGSQL_LOG_CHANNEL,"WARNING: you should run: UPDATE users SET userpass='%%' WHERE userpass is NULL\n");
       return uid; /* passworldless login */
     }
 
     if (strcmp(stored_pass,"%")==0)
       return uid; /* passworldless login */
 
-    cipher = (char*)md5_hash_r(pass, buffer, sizeof(buffer));
-    if (!cipher) return (uid_t)-1;
-
-    if (strncasecmp(cipher,stored_pass,32))
-      return (uid_t)-1;
+    if (check_auth(login, pass, stored_pass)==1)
+      return uid;
+    return (uid_t)-1;
 
   } /* else // user does not exist in table
     return -1;*/
@@ -370,6 +391,7 @@ wzd_user_t * FCN_GET_USER(uid_t uid)
   wzd_row_get_string(user->username, HARD_USERNAME_LENGTH, res, UCOL_USERNAME);
   wzd_row_get_string(user->userpass, MAX_PASS_LENGTH, res, UCOL_USERPASS);
   wzd_row_get_string(user->rootpath, WZD_MAX_PATH, res, UCOL_ROOTPATH);
+  wzd_row_get_string(user->tagline, MAX_TAGLINE_LENGTH, res, UCOL_TAGLINE);
   wzd_row_get_string(user->flags, MAX_FLAGS_NUM, res, UCOL_FLAGS);
   wzd_row_get_uint((unsigned int*)&user->max_idle_time, res, UCOL_MAX_IDLE_TIME);
   wzd_row_get_uint(&user->max_ul_speed, res, UCOL_MAX_UL_SPEED);
@@ -399,7 +421,7 @@ wzd_user_t * FCN_GET_USER(uid_t uid)
 
   for (i=0; (int)i<PQntuples(res); i++) {
     if (i >= HARD_IP_PER_USER) {
-      fprintf(stderr,"PGsql: too many IP for user %s, dropping others\n",user->username);
+      out_log(PGSQL_LOG_CHANNEL,"PGsql: too many IP for user %s, dropping others\n",user->username);
       break;
     }
     wzd_row_get_string_offset(user->ip_allowed[i], MAX_IP_LENGTH, res, i, 0 /* query asks only one column */);
@@ -422,7 +444,7 @@ wzd_user_t * FCN_GET_USER(uid_t uid)
 
   for (i=0; (int)i<PQntuples(res); i++) {
     if (i >= HARD_IP_PER_USER) {
-      fprintf(stderr,"PGsql: too many groups for user %s, dropping others\n",user->username);
+      out_log(PGSQL_LOG_CHANNEL,"PGsql: too many groups for user %s, dropping others\n",user->username);
       break;
     }
     if (wzd_row_get_uint(&j, res, 0 /* query asks only one column */)==0)
@@ -661,7 +683,8 @@ static uid_t * wzd_pgsql_get_user_list(void)
 {
   char *query;
   uid_t * uid_list;
-  int index, i;
+  int index;
+  unsigned int i;
   int num_rows;
   PGresult * res;
 
@@ -698,7 +721,8 @@ static gid_t * wzd_pgsql_get_group_list(void)
 {
   char *query;
   gid_t * gid_list;
-  int index, i;
+  int index;
+  unsigned int i;
   int num_rows;
   PGresult * res;
 
@@ -787,6 +811,7 @@ int _wzd_run_update_query(char * query, size_t length, const char * query_format
 
   if (!res || PQresultStatus(res) != PGRES_COMMAND_OK) {
     _wzd_pgsql_error(__FILE__, __FUNCTION__, __LINE__);
+    out_log(PGSQL_LOG_CHANNEL,"PG: query was '%s'\n", query);
     return -1;
   }
 
