@@ -37,7 +37,7 @@
 
 #include <stdio.h>
 
-#ifdef _MSC_VER
+#ifdef WIN32
 #include <winsock2.h>
 #include <direct.h>
 #include <io.h>
@@ -57,24 +57,29 @@
 #include <fcntl.h>
 #include <errno.h>
 
+#ifdef __MINGW32__
+# define _INTPTR_T_DEFINED
+# define _UINTPTR_T_DEFINED
+# undef fileno
+#endif
 
 #include <EXTERN.h>
 #include <perl.h>
 #include <XSUB.h>
 
-/*#include <wzd.h>*/
-#include "wzd_structs.h"
-#include "wzd_log.h"
-#include "wzd_misc.h"
-#include "wzd_libmain.h"
-#include "wzd_messages.h"
-#include "wzd_file.h" /* file_mkdir, file_stat */
-#include "wzd_vfs.h" /* checkpath_new */
-#include "wzd_mod.h" /* essential to define WZD_MODULE_INIT */
-#include "wzd_string.h"
-#include "wzd_vars.h" /* needed to access variables */
+#include <libwzd-core/wzd_structs.h>
+#include <libwzd-core/wzd_log.h>
+#include <libwzd-core/wzd_misc.h>
+#include <libwzd-core/wzd_configfile.h> /* server configuration */
+#include <libwzd-core/wzd_libmain.h>
+#include <libwzd-core/wzd_messages.h>
+#include <libwzd-core/wzd_file.h> /* file_mkdir, file_stat */
+#include <libwzd-core/wzd_vfs.h> /* checkpath_new */
+#include <libwzd-core/wzd_mod.h> /* essential to define WZD_MODULE_INIT */
+#include <libwzd-core/wzd_string.h>
+#include <libwzd-core/wzd_vars.h> /* needed to access variables */
 
-#include "wzd_debug.h"
+#include <libwzd-core/wzd_debug.h>
 
 
 #define PERL_ERRORLOGNAME "perlerr.log"
@@ -100,9 +105,10 @@ static int execute_perl( SV *function, const char *args);
 static void xs_init(pTHX);
 
 /***** EVENT HOOKS *****/
-static wzd_hook_reply_t perl_hook_site(unsigned long event_id, wzd_context_t * context, const char *token, const char *args);
 static int perl_hook_logout(unsigned long event_id, wzd_context_t *context, const char *username);
 
+static int do_site_perl(wzd_string_t *name, wzd_string_t *param, wzd_context_t *context);
+
 /***** PROTO HOOKS *****/
 static int perl_hook_protocol(const char *file, const char *args);
 
@@ -138,7 +144,7 @@ static struct _slave_t _slaves[MAX_SLAVES];
 
 /***********************/
 MODULE_NAME(perl);
-MODULE_VERSION(102);
+MODULE_VERSION(103);
 
 /***********************/
 /* WZD_MODULE_INIT     */
@@ -149,12 +155,22 @@ int WZD_MODULE_INIT(void)
     return -1;
 
   {
-    char * logdir;
+    char * logdir = NULL;
     int ret;
 
     ret = -1;
-    if (chtbl_lookup((CHTBL*)mainConfig->htab, "logdir", (void**)&logdir)== 0)
-    {
+    if (mainConfig->htab) {
+      chtbl_lookup((CHTBL*)mainConfig->htab, "logdir", (void**)&logdir);
+    } else { /* new config format */
+      wzd_string_t * str;
+      str = config_get_string(mainConfig->cfg_file, "GLOBAL", "logdir", NULL);
+      if (str) {
+        /** \bug FIXME memory leak here !! */
+        logdir = strdup(str_tochar(str));
+        str_deallocate(str);
+      }
+    }
+    if (logdir) {
       int fd;
 	  const char * filename;
 
@@ -164,7 +180,7 @@ int WZD_MODULE_INIT(void)
 #ifndef WIN32
       fd = open(filename,O_CREAT|O_WRONLY,S_IRUSR | S_IWUSR);
 #else
-	  /* activeperl redefines open(), and this causes a segfault here ! */
+      /* activeperl redefines open(), and this causes a segfault here ! */
       fd = _open(filename,O_CREAT|O_WRONLY,S_IRUSR | S_IWUSR);
 #endif
       if (fd >= 0) {
@@ -190,7 +206,20 @@ int WZD_MODULE_INIT(void)
   }
   memset(_slaves, 0, MAX_SLAVES*sizeof(struct _slave_t));
 
-  hook_add(&getlib_mainConfig()->hook,EVENT_SITE,(void_fct)&perl_hook_site);
+  {
+    const char * command_name = "site_perl";
+    /* add custom command */
+    if (commands_add(getlib_mainConfig()->commands_list,command_name,do_site_perl,NULL,TOK_CUSTOM)) {
+      out_log(LEVEL_HIGH,"ERROR while adding custom command: %s\n",command_name);
+    }
+
+    /* default permission XXX hardcoded */
+    if (commands_set_permission(getlib_mainConfig()->commands_list,command_name,"+O")) {
+      out_log(LEVEL_HIGH,"ERROR setting default permission to custom command %s\n",command_name);
+      /** \bug XXX remove command from   config->commands_list */
+    }
+  }
+
   hook_add(&getlib_mainConfig()->hook,EVENT_LOGOUT,(void_fct)&perl_hook_logout);
   hook_add_protocol("perl:",5,&perl_hook_protocol);
   out_log(LEVEL_INFO,"PERL module loaded\n");
@@ -210,37 +239,37 @@ void WZD_MODULE_CLOSE(void)
 }
 
 
-
-static wzd_hook_reply_t perl_hook_site(unsigned long event_id, wzd_context_t * context, const char *token, const char *args)
+static int do_site_perl(wzd_string_t *name, wzd_string_t *param, wzd_context_t *context)
 {
   SV *val;
 
-  if (strcasecmp(token,"site_perl")==0) {
-    if (!my_perl) return 0;
-    if (!args || strlen(args)==0) { do_perl_help(context); return EVENT_HANDLED; }
+  if (!my_perl) return 0;
+  if (!param || str_length(param)==0) { do_perl_help(context); return -1; }
 
-    if (_perl_set_slave(context)) return EVENT_ERROR;
-      
-    /* send reply header */
-    send_message_raw("200-\r\n",context);
-
-    /* exec string */
-    val = eval_pv(args, FALSE);
-
-    if (SvTRUE(val))
-      send_message_with_args(200,context,"PERL command ok");
-    else {
-      /* log error */
-      if (perl_fd_errlog >= 0) {
-        wzd_string_t * str = str_allocate();
-        str_sprintf(str,"Error in %s: %s\n",args,SvPV_nolen(ERRSV));
-        write(perl_fd_errlog,str_tochar(str),strlen(str_tochar(str)));
-      }
-      send_message_with_args(200,context,"PERL command reported errors");
-    }
-    return EVENT_HANDLED;
+  if (_perl_set_slave(context)) {
+    send_message_with_args(501,context,"Perl: could not set slave");
+    return -1;
   }
-  return EVENT_IGNORED;
+
+  /* send reply header */
+  send_message_raw("200-\r\n",context);
+
+  /* exec string */
+  val = eval_pv(str_tochar(param), FALSE);
+
+  if (SvTRUE(val))
+    send_message_with_args(200,context,"PERL command ok");
+  else {
+    /* log error */
+    if (perl_fd_errlog >= 0) {
+      wzd_string_t * str = str_allocate();
+      str_sprintf(str,"Error in %s: %s\n",str_tochar(param),SvPV_nolen(ERRSV));
+      write(perl_fd_errlog,str_tochar(str),strlen(str_tochar(str)));
+    }
+    send_message_with_args(200,context,"PERL command reported errors");
+  }
+
+  return 0;
 }
 
 /** \bug this code is not reentrant, be carefull with _perl_set_slave ! */
@@ -257,7 +286,7 @@ static int perl_hook_logout(unsigned long event_id, wzd_context_t * context, con
       perl_free(_slaves[i].interp);
       _slaves[i].context = NULL;
       _slaves[i].is_allocated = 1;
-      
+
       break;
     }
   }
@@ -348,7 +377,7 @@ static PerlInterpreter * perl_init(void)
 "  return 0;\n"
 "}\n"
   };
-  
+
   char * perl_args[] = { "", "-e", "0", "-w" };
   PerlInterpreter * interp = NULL;
 
@@ -480,7 +509,7 @@ static int _perl_set_slave(void *context)
       _slaves[i].interp = perl_clone(my_perl,0);
 #endif
       /* see perlapi (1) for more info, this flag is needed for win32 */
-      
+
       PERL_SET_CONTEXT(_slaves[i].interp);
 #else /* USE_ITHREADS */
       _slaves[i].is_allocated = 1;
@@ -1242,10 +1271,9 @@ static XS(XS_wzd_vfs)
   }
   else
     ret = 1;
-  
+
   if (!ret)
     XSRETURN_YES;
   else
     XSRETURN_NO;
 }
-
