@@ -29,7 +29,7 @@
 #include <string.h>
 #include <ctype.h> /* isdigit */
 
-#ifdef _MSC_VER
+#ifdef WIN32
 #include <winsock2.h>
 #include <direct.h>
 #include <io.h>
@@ -50,17 +50,19 @@
 #include <errno.h>
 
 /*#include "wzd.h"*/
-#include "wzd_structs.h"
-#include "wzd_log.h"
-#include "wzd_misc.h"
-#include "wzd_libmain.h"
-#include "wzd_messages.h"
-#include "wzd_mod.h" /* essential to define WZD_MODULE_INIT */
-#include "wzd_cache.h"
-#include "wzd_crc32.h"
-#include "wzd_vfs.h" /* checkabspath */
+#include <libwzd-core/wzd_structs.h>
+#include <libwzd-core/wzd_log.h>
+#include <libwzd-core/wzd_misc.h>
+#include <libwzd-core/wzd_events.h>
+#include <libwzd-core/wzd_libmain.h>
+#include <libwzd-core/wzd_messages.h>
+#include <libwzd-core/wzd_mod.h> /* essential to define WZD_MODULE_INIT */
+#include <libwzd-core/wzd_cache.h>
+#include <libwzd-core/wzd_configfile.h>
+#include <libwzd-core/wzd_crc32.h>
+#include <libwzd-core/wzd_vfs.h> /* checkabspath */
 
-#include "wzd_debug.h"
+#include <libwzd-core/wzd_debug.h>
 
 #include "libwzd_sfv.h"
 #include "libwzd_sfv_zip.h"
@@ -69,7 +71,7 @@
 
 /***********************/
 MODULE_NAME(sfv);
-MODULE_VERSION(109);
+MODULE_VERSION(110);
 
 static char progressmeter[256];
 static char del_progressmeter[256];
@@ -86,27 +88,50 @@ void sfv_update_completebar(wzd_sfv_file sfv, const char *filename, wzd_context_
 /* parse dir to calculate release completion % */
 float _sfv_get_release_percent(const char *dir, wzd_sfv_file sfv);
 
+
+static event_reply_t sfv_event_preupload(const char * args);
+int sfv_hook_preupload(unsigned long event_id, const char * username, const char *filename);
+static event_reply_t sfv_event_postupload(const char * args);
+int sfv_hook_postupload(unsigned long event_id, const char * username, const char *filename);
+
 /* get params from server */
 static int get_all_params(void)
 {
+  wzd_string_t * str;
+
   if (params_ok) return 0;
 
-  if (server_get_param("sfv_progressmeter",progressmeter,256,getlib_mainConfig()->param_list)) {
-    out_log(LEVEL_HIGH,"Module SFV: missing parameter 'sfv_progressmeter'\n");
+  str = config_get_string(getlib_mainConfig()->cfg_file,"sfv","progressmeter",NULL);
+  if (!str) {
+    out_log(LEVEL_HIGH,"Module SFV: missing parameter '[sfv] progressmeter'\n");
     return 1;
   }
-  if (server_get_param("sfv_del_progressmeter",del_progressmeter,256,getlib_mainConfig()->param_list)) {
-    out_log(LEVEL_HIGH,"Module SFV: missing parameter 'sfv_del_progressmeter'\n");
+  strncpy(progressmeter,str_tochar(str),sizeof(progressmeter));
+  str_deallocate(str);
+
+  str = config_get_string(getlib_mainConfig()->cfg_file,"sfv","del_progressmeter",NULL);
+  if (!str) {
+    out_log(LEVEL_HIGH,"Module SFV: missing parameter '[sfv] del_progressmeter'\n");
     return 1;
   }
-  if (server_get_param("sfv_incomplete_indicator",incomplete_indicator,256,getlib_mainConfig()->param_list)) {
-    out_log(LEVEL_HIGH,"Module SFV: missing parameter 'sfv_incomplete_indicator'\n");
+  strncpy(del_progressmeter,str_tochar(str),sizeof(del_progressmeter));
+  str_deallocate(str);
+
+  str = config_get_string(getlib_mainConfig()->cfg_file,"sfv","incomplete_indicator",NULL);
+  if (!str) {
+    out_log(LEVEL_HIGH,"Module SFV: missing parameter '[sfv] incomplete_indicator'\n");
     return 1;
   }
-  if (server_get_param("sfv_other_completebar",other_completebar,256,getlib_mainConfig()->param_list)) {
-    out_log(LEVEL_HIGH,"Module SFV: missing parameter 'sfv_other_completebar'\n");
+  strncpy(incomplete_indicator,str_tochar(str),sizeof(incomplete_indicator));
+  str_deallocate(str);
+
+  str = config_get_string(getlib_mainConfig()->cfg_file,"sfv","other_completebar",NULL);
+  if (!str) {
+    out_log(LEVEL_HIGH,"Module SFV: missing parameter '[sfv] other_completebar'\n");
     return 1;
   }
+  strncpy(other_completebar,str_tochar(str),sizeof(other_completebar));
+  str_deallocate(str);
 
   params_ok = 1;
   return 0;
@@ -284,7 +309,7 @@ int sfv_create(const char * sfv_file)
   int i;
   unsigned long crc;
   struct stat s;
-#ifndef _MSC_VER
+#ifndef WIN32
   DIR *dir;
   struct dirent *entr;
 #else
@@ -308,14 +333,14 @@ int sfv_create(const char * sfv_file)
   *(++ptr) = '\0';
 
   strcpy(filename,directory);
-#ifndef _MSC_VER
+#ifndef WIN32
   if ((dir=opendir(directory))==NULL) return -1;
 #else
   _snprintf(dirfilter,2048,"%s/*",directory);
   if ((dir = FindFirstFile(dirfilter,&fileData))== INVALID_HANDLE_VALUE) return -1;
 #endif
 
-#ifndef _MSC_VER
+#ifndef WIN32
   while ((entr=readdir(dir))!=NULL) {
     dir_filename = entr->d_name;
 #else
@@ -357,7 +382,11 @@ int sfv_create(const char * sfv_file)
     count_entries++;
 	DIR_CONTINUE
   } /* while ((entr=readdir(dir))!=NULL) */
+#ifndef WIN32
   closedir(dir);
+#else
+  FindClose(dir);
+#endif
   sfv.comments[count_comments] = NULL;
   sfv.sfv_list[count_entries] = NULL;
 
@@ -380,7 +409,11 @@ int sfv_create(const char * sfv_file)
       ret = strlen(buffer);
       if ( write(fd_sfv,buffer,ret) != ret ) {
 	out_err(LEVEL_CRITICAL,"Unable to write sfv_file (%s)\n",strerror(errno));
+#ifndef WIN32
         closedir(dir);
+#else
+        FindClose(dir);
+#endif
        	return -1;
       }
       i++;
@@ -460,7 +493,7 @@ out_err(LEVEL_CRITICAL,"file %s calculated: %08lX reference: %08lX\n",filename,c
  */
 int sfv_find_sfv(const char * file, wzd_sfv_file *sfv, wzd_sfv_entry ** entry)
 {
-#ifndef _MSC_VER
+#ifndef WIN32
   DIR *dir;
   struct dirent *entr;
 #else
@@ -485,7 +518,7 @@ int sfv_find_sfv(const char * file, wzd_sfv_file *sfv, wzd_sfv_entry ** entry)
   strncpy(stripped_filename,ptr+1,1023);
   if (strlen(stripped_filename)<=0) return -1;
 
-#ifndef _MSC_VER
+#ifndef WIN32
   if ( (dir=opendir(sfv_dir)) == NULL ) return -1;
 #else
   _snprintf(dirfilter,2048,"%s/*",sfv_dir);
@@ -494,7 +527,7 @@ int sfv_find_sfv(const char * file, wzd_sfv_file *sfv, wzd_sfv_entry ** entry)
 
   sfv_init(sfv);
 
-#ifndef _MSC_VER
+#ifndef WIN32
   while ( (entr=readdir(dir)) != NULL ) {
     dir_filename = entr->d_name;
 #else
@@ -519,7 +552,7 @@ int sfv_find_sfv(const char * file, wzd_sfv_file *sfv, wzd_sfv_entry ** entry)
 	sfv_name[i] = *ptr;
 	i++;
 	ptr++;
-      }	
+      }
       sfv_name[i++] = '/';
       ptr = dir_filename;
       while (*ptr) {
@@ -527,14 +560,21 @@ int sfv_find_sfv(const char * file, wzd_sfv_file *sfv, wzd_sfv_entry ** entry)
 	sfv_name[i] = *ptr;
 	i++;
 	ptr++;
-      }	
+      }
       *ptr = '\0';
       sfv_name[i]='\0';
       ret = sfv_read(sfv_name,sfv);
 #ifdef DEBUG
       out_err(LEVEL_CRITICAL,"sfv file: %s\n",entr->d_name);
 #endif
-      if (ret == -1 || sfv->sfv_list == NULL) { closedir(dir); return -1; }
+      if (ret == -1 || sfv->sfv_list == NULL) {
+#ifndef WIN32
+        closedir(dir);
+#else
+        FindClose(dir);
+#endif
+		  return -1;
+	  }
       /* sfv file found, check if file is in sfv */
       i = 0;
       while (sfv->sfv_list[i]) {
@@ -544,7 +584,11 @@ int sfv_find_sfv(const char * file, wzd_sfv_file *sfv, wzd_sfv_entry ** entry)
 	if (strcmp(stripped_filename,sfv->sfv_list[i]->filename)==0) {
 #endif /* WIN32 */
 	  *entry = sfv->sfv_list[i];
-	  closedir(dir);
+#ifndef WIN32
+      closedir(dir);
+#else
+      FindClose(dir);
+#endif
 	  return 0;
 	}
 	i++;
@@ -554,7 +598,11 @@ int sfv_find_sfv(const char * file, wzd_sfv_file *sfv, wzd_sfv_entry ** entry)
 	DIR_CONTINUE
   } /* while readdir */
 
+#ifndef WIN32
   closedir(dir);
+#else
+  FindClose(dir);
+#endif
 
   return 1;
 }
@@ -750,7 +798,7 @@ const char *_sfv_convert_cookies(char * instr, const char *dir, wzd_sfv_file sfv
 			from = to = reverse = 0;
 			instr++;
 			m = instr;
-			if ( *instr == '-' ) {  
+			if ( *instr == '-' ) {
 				reverse = 1;
 				instr++;
 				}
@@ -762,7 +810,7 @@ const char *_sfv_convert_cookies(char * instr, const char *dir, wzd_sfv_file sfv
 
 			if ( *instr == '-' ) {
 				instr++;
-				for ( ; isdigit(*instr) ; instr++ ) { 
+				for ( ; isdigit(*instr) ; instr++ ) {
 					to *= 10;
 					to += *instr - 48;
 					}
@@ -794,7 +842,7 @@ const char *_sfv_convert_cookies(char * instr, const char *dir, wzd_sfv_file sfv
 			from = to = reverse = 0;
 			instr++;
 			m = instr;
-			if ( *instr == '-' ) {  
+			if ( *instr == '-' ) {
 				reverse = 1;
 				instr++;
 				}
@@ -806,7 +854,7 @@ const char *_sfv_convert_cookies(char * instr, const char *dir, wzd_sfv_file sfv
 
 			if ( *instr == '-' ) {
 				instr++;
-				for ( ; isdigit(*instr) ; instr++ ) { 
+				for ( ; isdigit(*instr) ; instr++ ) {
 					to *= 10;
 					to += *instr - 48;
 					}
@@ -915,11 +963,11 @@ void sfv_update_completebar(wzd_sfv_file sfv, const char *filename, wzd_context_
   len = (ptr-filename)+1; /* +1 because we want the / */
   strncpy(dir,filename,len);
   dir[len]='\0';
-  
+
   regcomp( &preg, del_progressmeter, REG_NEWLINE|REG_EXTENDED );
   {
     char buffer[512];
-#ifndef _MSC_VER
+#ifndef WIN32
     DIR *d;
     struct dirent *entr;
 #else
@@ -932,13 +980,13 @@ void sfv_update_completebar(wzd_sfv_file sfv, const char *filename, wzd_context_
     float percent;
 
     /* Removes previous progressmeter */
-#ifndef _MSC_VER
+#ifndef WIN32
     if ( (d=opendir(dir))==NULL ) return;
 #else
     snprintf(dirfilter,2048,"%s/*",dir);
     if ((d = FindFirstFile(dirfilter,&fileData))== INVALID_HANDLE_VALUE) return;
 #endif
-#ifndef _MSC_VER
+#ifndef WIN32
     while ((entr=readdir(d))!=NULL) {
       dir_filename = entr->d_name;
 #else
@@ -948,7 +996,7 @@ void sfv_update_completebar(wzd_sfv_file sfv, const char *filename, wzd_context_
 #endif
       if (dir_filename[0]=='.')
 	  {
-#ifdef _MSC_VER
+#ifdef WIN32
         if (!FindNextFile(d,&fileData))
 		{
 	      if (GetLastError() == ERROR_NO_MORE_FILES)
@@ -962,7 +1010,7 @@ void sfv_update_completebar(wzd_sfv_file sfv, const char *filename, wzd_context_
 	/* security check */
 	if (len+strlen(dir_filename)>510)
 	  {
-#ifdef _MSC_VER
+#ifdef WIN32
         if (!FindNextFile(d,&fileData))
 		{
 	      if (GetLastError() == ERROR_NO_MORE_FILES)
@@ -975,7 +1023,7 @@ void sfv_update_completebar(wzd_sfv_file sfv, const char *filename, wzd_context_
 	rmdir (dir);
 	dir[len]='\0';
       }
-#ifdef _MSC_VER
+#ifdef WIN32
     if (!FindNextFile(d,&fileData))
 	{
       if (GetLastError() == ERROR_NO_MORE_FILES)
@@ -983,7 +1031,11 @@ void sfv_update_completebar(wzd_sfv_file sfv, const char *filename, wzd_context_
 	}
 #endif
     }
+#ifndef WIN32
     closedir(d);
+#else
+    FindClose(d);
+#endif
 
     percent = _sfv_get_release_percent(dir, sfv);
     if (percent >= 100.f) { /* complete */
@@ -1024,9 +1076,16 @@ void sfv_update_completebar(wzd_sfv_file sfv, const char *filename, wzd_context_
 	  buffer[len++]='/';
 	  buffer[len]='\0';
 	}
-	strncpy(buffer+len,context->last_command+5,2048-len);
+	strncpy(buffer+len,context->current_action.arg,2048-len);
 	ptr = strrchr(buffer,'/');
-	if (!ptr) { closedir(d); return; }
+	if (!ptr) {
+#ifndef WIN32
+      closedir(d);
+#else
+      FindClose(d);
+#endif
+	  return;
+	}
 	*ptr='\0';
 	if (user->group_num>0) {
 	  wzd_group_t * group;
@@ -1061,7 +1120,7 @@ float _sfv_get_release_percent(const char *dir, wzd_sfv_file sfv)
   struct stat s;
 
   if (sfv.sfv_list == NULL) return 0;
-  
+
   strncpy(buffer,dir,511);
   len = strlen(buffer);
   if (buffer[len-1] != '/') { buffer[len-1]='/'; len++; }
@@ -1303,31 +1362,35 @@ int sfv_process_diz(const char *diz_file, wzd_context_t *context)
  * check sfv_name
  * create new_sfv_name
  */
-void do_site_sfv(char *command_line, wzd_context_t * context)
+/*void do_site_sfv(char *command_line, wzd_context_t * context)*/
+static int do_site_sfv(wzd_string_t *commandname, wzd_string_t *param, wzd_context_t *context)
 {
   char buffer[BUFFER_LEN];
   char * ptr;
   char * command, *name;
   int ret;
+  char * command_line;
   wzd_sfv_file sfv;
 
+  command_line = (char*)str_tochar(param); /** \todo convert code to use wzd_string_t */
+
   ptr = command_line;
   command = strtok_r(command_line," \t\r\n",&ptr);
   if (!command) {
     do_site_help_sfv(context);
-    return;
+    return -1;
   }
   name = strtok_r(NULL," \t\r\n",&ptr);
 
   if (!name) {
     do_site_help_sfv(context);
-    return;
+    return -1;
   }
 
   /* convert file to absolute path, remember sfv wants ABSOLUTE paths ! */
-  if ( (ret = checkpath(name,buffer,context)) != 0 ) {
+  if ( (ret = checkpath_new(name,buffer,context)) != 0 ) {
     do_site_help_sfv(context);
-    return;
+    return -1;
   }
 /*  buffer[strlen(buffer)-1] = '\0';*/ /* remove '/', appended by checkpath */
   sfv_init(&sfv);
@@ -1356,12 +1419,30 @@ void do_site_sfv(char *command_line, wzd_context_t * context)
        ret = send_message_with_args(501,context,"Critical error occured");
     }
   }
-  
+
   sfv_free(&sfv);
+
+  return ret;
 }
 
 
 /***** EVENT HOOKS *****/
+static event_reply_t sfv_event_preupload(const char * args)
+{
+  int ret;
+  const char * username, * filename;
+  char * str = strdup(args), * ptr;
+
+  username = strtok_r(str," ",&ptr);
+  filename = ptr;
+
+  ret = sfv_hook_preupload(EVENT_PREUPLOAD, username, filename);
+
+  free(str);
+
+  return EVENT_OK;
+}
+
 int sfv_hook_preupload(unsigned long event_id, const char * username, const char *filename)
 {
   wzd_sfv_file sfv;
@@ -1395,6 +1476,22 @@ int sfv_hook_preupload(unsigned long event_id, const char * username, const char
   return 0;
 }
 
+static event_reply_t sfv_event_postupload(const char * args)
+{
+  int ret;
+  const char * username, * filename;
+  char * str = strdup(args), * ptr;
+
+  username = strtok_r(str," ",&ptr);
+  filename = ptr;
+
+  ret = sfv_hook_postupload(EVENT_POSTUPLOAD, username, filename);
+
+  free(str);
+
+  return EVENT_OK;
+}
+
 int sfv_hook_postupload(unsigned long event_id, const char * username, const char *filename)
 {
   wzd_sfv_file sfv;
@@ -1447,28 +1544,29 @@ int sfv_hook_postupload(unsigned long event_id, const char * username, const cha
   return ret;
 }
 
-int sfv_hook_site(unsigned long event_id, wzd_context_t * context, const char *token, const char *args)
-{
-  if (strcasecmp(token,"site_sfv")==0) {
-    char buffer[BUFFER_LEN];
-    strncpy(buffer,args,BUFFER_LEN-1);
-    do_site_sfv(buffer,context);
-    return EVENT_HANDLED;
-  }
-
-  return EVENT_IGNORED;
-}
-
 /***********************/
 /* WZD_MODULE_INIT     */
 
 int WZD_MODULE_INIT(void)
 {
 /*  printf("WZD_MODULE_INIT\n");*/
-/*  out_err(LEVEL_INFO,"max threads: %d\n",getlib_mainConfig()->max_threads);*/
-  hook_add(&getlib_mainConfig()->hook,EVENT_PREUPLOAD,(void_fct)&sfv_hook_preupload);
-  hook_add(&getlib_mainConfig()->hook,EVENT_POSTUPLOAD,(void_fct)&sfv_hook_postupload);
-  hook_add(&getlib_mainConfig()->hook,EVENT_SITE,(void_fct)&sfv_hook_site);
+  event_connect_function(getlib_mainConfig()->event_mgr,EVENT_PREUPLOAD,sfv_event_preupload,NULL);
+  event_connect_function(getlib_mainConfig()->event_mgr,EVENT_POSTUPLOAD,sfv_event_postupload,NULL);
+
+  {
+    const char * command_name = "site_sfv";
+    /* add custom command */
+    if (commands_add(getlib_mainConfig()->commands_list,command_name,do_site_sfv,NULL,TOK_CUSTOM)) {
+      out_log(LEVEL_HIGH,"ERROR while adding custom command: %s\n",command_name);
+    }
+
+    /* default permission XXX hardcoded */
+    if (commands_set_permission(getlib_mainConfig()->commands_list,command_name,"+O")) {
+      out_log(LEVEL_HIGH,"ERROR setting default permission to custom command %s\n",command_name);
+      /** \bug XXX remove command from   config->commands_list */
+    }
+  }
+
 #ifdef DEBUG
   out_err(LEVEL_INFO,"module sfv: hooks registered\n");
 #endif
@@ -1477,9 +1575,9 @@ int WZD_MODULE_INIT(void)
 
 int WZD_MODULE_CLOSE(void)
 {
-  hook_remove(&getlib_mainConfig()->hook,EVENT_PREUPLOAD,(void_fct)&sfv_hook_preupload);
+/*  hook_remove(&getlib_mainConfig()->hook,EVENT_PREUPLOAD,(void_fct)&sfv_hook_preupload);
   hook_remove(&getlib_mainConfig()->hook,EVENT_POSTUPLOAD,(void_fct)&sfv_hook_postupload);
-  hook_remove(&getlib_mainConfig()->hook,EVENT_SITE,(void_fct)&sfv_hook_site);
+  hook_remove(&getlib_mainConfig()->hook,EVENT_SITE,(void_fct)&sfv_hook_site);*/
 #ifdef DEBUG
   out_err(LEVEL_INFO,"module sfv: hooks unregistered\n");
 #endif
